@@ -24,7 +24,6 @@
 #include "debug.h"
 #include "fileinfo.h"
 #include "dir.h"
-#include "growbuf.h"
 #include "entry.h"
 #include "commentscan.h"
 #include "linkedmap.h"
@@ -101,8 +100,7 @@ void CitationManager::insert(const QCString &label)
 
 const CiteInfo *CitationManager::find(const QCString &label) const
 {
-  auto it = p->entries.find(label.lower().str());
-  if (it!=p->entries.end())
+  if (auto it = p->entries.find(label.lower().str()); it != p->entries.end())
   {
     return it->second.get();
   }
@@ -233,17 +231,17 @@ void CitationManager::insertCrossReferencesForBibFile(const QCString &bibFile)
   }
 }
 
-const std::string g_formulaMarker = "CITE_FORMULA_";
+static const std::string g_formulaMarker = "CITE_FORMULA_";
 
 QCString CitationManager::getFormulas(const QCString &s)
 {
   if (s.isEmpty()) return s;
-  GrowBuf growBuf;
-  GrowBuf formulaBuf;
+  QCString result;
+  result.reserve(s.length()+32);
+  QCString formula;
+  formula.reserve(256);
   bool insideFormula = false;
   int citeFormulaCnt = 1;
-  const size_t tmpLen = 30;
-  char tmp[tmpLen];
   const char *ps=s.data();
   char c = 0;
   while ((c=*ps++))
@@ -253,32 +251,34 @@ QCString CitationManager::getFormulas(const QCString &s)
       switch (c)
       {
         case '\\':
-          formulaBuf.addChar(c);
+          formula+=c;
           c = *ps++;
-          formulaBuf.addChar(c);
+          formula+=c;
           break;
         case '\n':
-          formulaBuf.addChar(c);
-          formulaBuf.addChar(0);
-          growBuf.addChar('$');
-          growBuf.addStr(formulaBuf.get());
+          formula+=c;
+          result+='$';
+          result+=formula;
           insideFormula = false;
-          formulaBuf.clear();
+          formula.clear();
           break;
         case '$':
-          qsnprintf(tmp,tmpLen,"%s%06d",g_formulaMarker.c_str(),citeFormulaCnt);
-          formulaBuf.addChar(0);
-          p->formulaCite.emplace(citeFormulaCnt,std::string("\\f$") + formulaBuf.get() + "\\f$");
-          citeFormulaCnt++;
-          // need { and } due to the capitalization rules of bibtex.
-          growBuf.addChar('{');
-          growBuf.addStr(tmp);
-          growBuf.addChar('}');
-          insideFormula = false;
-          formulaBuf.clear();
+          {
+            const size_t idLen = 30;
+            char id[idLen];
+            qsnprintf(id,idLen,"%s%06d",g_formulaMarker.c_str(),citeFormulaCnt);
+            p->formulaCite.emplace(citeFormulaCnt,std::string("\\f$") + formula.str() + "\\f$");
+            citeFormulaCnt++;
+            // need { and } due to the capitalization rules of bibtex.
+            result+='{';
+            result+=id;
+            result+='}';
+            insideFormula = false;
+            formula.clear();
+          }
           break;
         default:
-          formulaBuf.addChar(c);
+          formula+=c;
           break;
       }
     }
@@ -287,27 +287,25 @@ QCString CitationManager::getFormulas(const QCString &s)
       switch (c)
       {
         case '\\':
-          growBuf.addChar(c);
+          result+=c;
           c = *ps++;
-          growBuf.addChar(c);
+          result+=c;
           break;
         case '$':
           insideFormula = true;
           break;
         default:
-          growBuf.addChar(c);
+          result+=c;
           break;
       }
     }
   }
   if (insideFormula)
   {
-    formulaBuf.addChar(0);
-    growBuf.addStr(formulaBuf.get());
-    formulaBuf.clear();
+    result+=formula;
+    formula.clear();
   }
-  growBuf.addChar(0);
-  return growBuf.get();
+  return result;
 }
 
 QCString CitationManager::replaceFormulas(const QCString &s)
@@ -316,7 +314,7 @@ QCString CitationManager::replaceFormulas(const QCString &s)
   QCString t;
   int pos=0;
   int i = -1;
-  while ((i=s.find(g_formulaMarker.c_str(),pos))!=-1)
+  while ((i=s.find(g_formulaMarker,pos))!=-1)
   {
     t += s.mid(pos,i-pos);
     int markerSize = static_cast<int>( g_formulaMarker.length());
@@ -343,7 +341,7 @@ void CitationManager::generatePage()
   const StringVector &citeDataList = Config_getList(CITE_BIB_FILES);
   for (const auto &bibdata : citeDataList)
   {
-    QCString bibFile = getBibFile(QCString(bibdata));
+    QCString bibFile = getBibFile(bibdata);
     insertCrossReferencesForBibFile(bibFile);
   }
 
@@ -392,7 +390,7 @@ void CitationManager::generatePage()
   int i = 0;
   for (const auto &bibdata : citeDataList)
   {
-    QCString bibFile = getBibFile(QCString(bibdata));
+    QCString bibFile = getBibFile(bibdata);
     FileInfo fi(bibFile.str());
     if (fi.exists())
     {
@@ -474,12 +472,12 @@ void CitationManager::generatePage()
           size_t uk=static_cast<size_t>(k1);
           QCString label = line.mid(ui+14,uj0-ui-14);
           StringVector optList = split(line.mid(uj+5,uk-uj-5).str(),",");
-          QCString number = optList[0].c_str();
-          QCString shortAuthor = optList[1].c_str();
+          QCString number = optList[0];
+          QCString shortAuthor = optList[1];
           QCString year;
           if (optList.size() == 3)
           {
-            year = optList[2].c_str();
+            year = optList[2];
           }
           line = line.left(ui+14) + label + line.right(line.length()-uj0);
           auto it = p->entries.find(label.lower().str());
@@ -537,7 +535,7 @@ void CitationManager::generatePage()
     i = 0;
     for (const auto &bibdata : citeDataList)
     {
-      QCString bibFile = getBibFile(QCString(bibdata));
+      QCString bibFile = getBibFile(bibdata);
       FileInfo fi(bibFile.str());
       if (fi.exists())
       {
@@ -581,17 +579,14 @@ QCString CitationManager::latexBibFiles()
   int i = 0;
   for (const auto &bibdata : citeDataList)
   {
-    QCString bibFile = getBibFile(QCString(bibdata));
+    QCString bibFile = getBibFile(bibdata);
     FileInfo fi(bibFile.str());
-    if (fi.exists())
+    if (fi.exists() && !bibFile.isEmpty())
     {
-      if (!bibFile.isEmpty())
-      {
-        if (i) result += ",";
-        i++;
-        result += bibTmpFile;
-        result += QCString().setNum(i);
-      }
+      if (i) result += ",";
+      i++;
+      result += bibTmpFile;
+      result += QCString().setNum(i);
     }
   }
   return result;
