@@ -20,6 +20,7 @@
 
 #include <vector>
 #include <memory>
+#include <optional>
 
 #include <sys/types.h>
 
@@ -40,31 +41,26 @@ struct TagInfo;
 class MemberDefMutable;
 class MemberGroupList;
 class MemberVector;
+class ModuleDef;
 
 /** A model of a class/file/namespace member symbol. */
 class MemberDef : public Definition
 {
   public:
-    virtual ~MemberDef() {}
-    virtual DefType definitionType() const = 0;
+    ABSTRACT_BASE_CLASS(MemberDef)
+
     // move this member into a different scope
-    virtual MemberDef *deepCopy() const =0;
+    virtual std::unique_ptr<MemberDef> deepCopy() const =0;
     virtual void moveTo(Definition *) = 0;
 
     virtual MemberDef *resolveAlias() = 0;
     virtual const MemberDef *resolveAlias() const = 0;
 
-    ClassDefMutable *getClassDefMutable() const;
 
     //-----------------------------------------------------------------------------------
     // ----  getters -----
     //-----------------------------------------------------------------------------------
 
-    // link id
-    virtual QCString getOutputFileBase() const = 0;
-    virtual QCString getReference() const = 0;
-    virtual QCString anchor() const = 0;
-
     virtual QCString declaration() const = 0;
     virtual QCString definition() const = 0;
     virtual QCString typeString() const = 0;
@@ -74,14 +70,20 @@ class MemberDef : public Definition
     virtual QCString extraTypeChars() const = 0;
     virtual const QCString &initializer() const = 0;
     virtual int initializerLines() const = 0;
-    virtual uint64 getMemberSpecifiers() const = 0;
+    virtual TypeSpecifier getMemberSpecifiers() const = 0;
+    virtual VhdlSpecifier getVhdlSpecifiers() const = 0;
     virtual const MemberList *getSectionList(const Definition *container) const = 0;
     virtual QCString    displayDefinition() const = 0;
 
     // scope query members
-    virtual const FileDef *getFileDef() const = 0;
-    virtual const ClassDef *getClassDef() const = 0;
+    virtual const FileDef *     getFileDef() const      = 0;
+    virtual       FileDef *     getFileDef()            = 0;
+    virtual const ClassDef *    getClassDef() const     = 0;
+    virtual       ClassDef *    getClassDef()           = 0;
     virtual const NamespaceDef* getNamespaceDef() const = 0;
+    virtual       NamespaceDef* getNamespaceDef()       = 0;
+    virtual const ModuleDef*    getModuleDef() const    = 0;
+
     virtual const ClassDef *accessorClass() const = 0;
 
     // grabbing the property read/write accessor names
@@ -89,13 +91,13 @@ class MemberDef : public Definition
     virtual QCString getWriteAccessor() const = 0;
 
     // querying the grouping definition
+    virtual GroupDef *getGroupDef() = 0;
     virtual const GroupDef *getGroupDef() const = 0;
     virtual Grouping::GroupPri_t getGroupPri() const = 0;
     virtual QCString getGroupFileName() const = 0;
     virtual int getGroupStartLine() const = 0;
     virtual bool getGroupHasDocs() const = 0;
 
-    virtual QCString qualifiedName() const = 0;
     virtual QCString objCMethodName(bool localLink,bool showStatic) const = 0;
 
     // direct kind info
@@ -126,6 +128,7 @@ class MemberDef : public Definition
     virtual bool isInline() const = 0;
     virtual bool isExplicit() const = 0;
     virtual bool isMutable() const = 0;
+    virtual bool isThreadLocal() const = 0;
     virtual bool isGettable() const = 0;
     virtual bool isPrivateGettable() const = 0;
     virtual bool isProtectedGettable() const = 0;
@@ -177,11 +180,14 @@ class MemberDef : public Definition
     virtual bool isDestructor() const = 0;
     virtual bool hasOneLineInitializer() const = 0;
     virtual bool hasMultiLineInitializer() const = 0;
-    virtual bool showInCallGraph() const = 0;
+    virtual bool isCallable() const = 0;
     virtual bool isStrongEnumValue() const = 0;
     virtual bool livesInsideEnum() const = 0;
     virtual bool isSliceLocal() const = 0;
     virtual bool isConstExpr() const = 0;
+    virtual bool isConstEval() const = 0;
+    virtual bool isConstInit() const = 0;
+    virtual bool isNoDiscard() const = 0;
     virtual int  numberOfFlowKeyWords() const = 0;
 
     // derived getters
@@ -191,10 +197,6 @@ class MemberDef : public Definition
     virtual bool isRelatedOrFriend() const = 0;
 
     // output info
-    virtual bool isLinkableInProject() const = 0;
-    virtual bool isLinkable() const = 0;
-    virtual bool hasDocumentation() const = 0;  // overrides hasDocumentation in definition.h
-    //bool hasUserDocumentation() const = 0; // overrides hasUserDocumentation
     virtual bool isDeleted() const = 0;
     virtual bool isBriefSectionVisible() const = 0;
     virtual bool isDetailedSectionVisible(MemberListContainer container) const = 0;
@@ -224,23 +226,27 @@ class MemberDef : public Definition
     virtual const ArgumentList &declArgumentList() const = 0;
     virtual const ArgumentList &templateArguments() const = 0;
     virtual const ArgumentLists &definitionTemplateParameterLists() const = 0;
+    virtual std::optional<ArgumentList> formalTemplateArguments() const = 0;
 
     // member group related members
     virtual int getMemberGroupId() const = 0;
     virtual MemberGroup *getMemberGroup() const = 0;
 
     virtual bool fromAnonymousScope() const = 0;
-    virtual bool anonymousDeclShown() const = 0;
     virtual MemberDef *fromAnonymousMember() const = 0;
 
     // callgraph related members
     virtual bool hasCallGraph() const = 0;
     virtual bool hasCallerGraph() const = 0;
-    virtual bool visibleMemberGroup(bool hideNoHeader) const = 0;
     // referenced related members
     virtual bool hasReferencesRelation() const = 0;
     virtual bool hasReferencedByRelation() const = 0;
 
+    virtual bool hasInlineSource() const = 0;
+    virtual bool hasEnumValues() const = 0;
+
+    virtual QCString sourceRefName() const = 0;
+
     virtual const MemberDef *templateMaster() const = 0;
     virtual QCString getScopeString() const = 0;
     virtual ClassDef *getClassDefOfAnonymousType() const = 0;
@@ -259,35 +265,35 @@ class MemberDef : public Definition
     virtual ClassDef *category() const = 0;
     virtual const MemberDef *categoryRelation() const = 0;
 
-    virtual QCString displayName(bool=TRUE) const = 0;
     virtual QCString getDeclType() const = 0;
     virtual StringVector getLabels(const Definition *container) const = 0;
+    virtual StringVector getQualifiers() const = 0;
 
     virtual const ArgumentList &typeConstraints() const = 0;
 
     virtual QCString requiresClause() const = 0;
 
-    // overrules
-    virtual QCString documentation() const = 0;
-    virtual QCString briefDescription(bool abbr=FALSE) const = 0;
     virtual QCString fieldType() const = 0;
-    virtual bool isReference() const = 0;
 
     virtual QCString getDeclFileName() const = 0;
     virtual int getDeclLine() const = 0;
     virtual int getDeclColumn() const = 0;
 
-    virtual MemberDefMutable *createTemplateInstanceMember(const ArgumentList &formalArgs,
+    virtual std::unique_ptr<MemberDef> createTemplateInstanceMember(const ArgumentList &formalArgs,
                const std::unique_ptr<ArgumentList> &actualArgs) const = 0;
     virtual void writeDeclaration(OutputList &ol,
-                 const ClassDef *cd,const NamespaceDef *nd,const FileDef *fd,const GroupDef *gd,
-                 bool inGroup, int indentLevel=0, const ClassDef *inheritFrom=0,const QCString &inheritId=QCString()) const = 0;
+                 const ClassDef *cd,const NamespaceDef *nd,const FileDef *fd,const GroupDef *gd,const ModuleDef *md,
+                 bool inGroup, int indentLevel=0, const ClassDef *inheritFrom=nullptr,const QCString &inheritId=QCString()) const = 0;
     virtual void writeEnumDeclaration(OutputList &typeDecl, const ClassDef *cd,
-                const NamespaceDef *nd,const FileDef *fd,const GroupDef *gd) const = 0;
+                const NamespaceDef *nd,const FileDef *fd,const GroupDef *gd,const ModuleDef *mod) const = 0;
+    virtual void writeLink(OutputList &ol,
+                 const ClassDef *cd,const NamespaceDef *nd,const FileDef *fd,const GroupDef *gd,const ModuleDef *md,
+                 bool onlyText=FALSE) const = 0;
     virtual void detectUndocumentedParams(bool hasParamCommand,bool hasReturnCommand) const = 0;
     virtual void warnIfUndocumented() const = 0;
     virtual void warnIfUndocumentedParams() const = 0;
     virtual bool visibleInIndex() const = 0;
+    virtual int redefineCount() const = 0;
 
     // TODO: this is not a getter, should be passed at construction
     virtual void setMemberGroup(MemberGroup *grp) = 0;
@@ -297,6 +303,7 @@ class MemberDef : public Definition
 class MemberDefMutable : public DefinitionMutable, public MemberDef
 {
   public:
+    ABSTRACT_BASE_CLASS(MemberDefMutable)
 
     //-----------------------------------------------------------------------------------
     // ----  setters -----
@@ -305,19 +312,20 @@ class MemberDefMutable : public DefinitionMutable, public MemberDef
     // set functions
     virtual void setMemberType(MemberType t) = 0;
     virtual void setDefinition(const QCString &d) = 0;
-    virtual void setFileDef(const FileDef *fd) = 0;
+    virtual void setFileDef(FileDef *fd) = 0;
     virtual void setAnchor() = 0;
     virtual void setProtection(Protection p) = 0;
-    virtual void setMemberSpecifiers(uint64 s) = 0;
-    virtual void mergeMemberSpecifiers(uint64 s) = 0;
+    virtual void setMemberSpecifiers(TypeSpecifier s) = 0;
+    virtual void setVhdlSpecifiers(VhdlSpecifier s) = 0;
+    virtual void mergeMemberSpecifiers(TypeSpecifier s) = 0;
     virtual void setInitializer(const QCString &i) = 0;
     virtual void setBitfields(const QCString &s) = 0;
     virtual void setMaxInitLines(int lines) = 0;
-    virtual void setMemberClass(const ClassDef *cd) = 0;
+    virtual void setMemberClass(ClassDef *cd) = 0;
     virtual void setSectionList(const Definition *container,const MemberList *sl) = 0;
-    virtual void setGroupDef(const GroupDef *gd,Grouping::GroupPri_t pri,
+    virtual void setGroupDef(GroupDef *gd,Grouping::GroupPri_t pri,
                      const QCString &fileName,int startLine,bool hasDocs,
-                     MemberDef *member=0) = 0;
+                     MemberDef *member=nullptr) = 0;
     virtual void setReadAccessor(const QCString &r) = 0;
     virtual void setWriteAccessor(const QCString &w) = 0;
     virtual void setTemplateSpecialization(bool b) = 0;
@@ -331,15 +339,15 @@ class MemberDefMutable : public DefinitionMutable, public MemberDef
     virtual void setEnumBaseType(const QCString &type) = 0;
 
     // relation to other members
-    virtual void setReimplements(const MemberDef *md) = 0;
-    virtual void insertReimplementedBy(const MemberDef *md) = 0;
+    virtual void setReimplements(MemberDef *md) = 0;
+    virtual void insertReimplementedBy(MemberDef *md) = 0;
 
     virtual void setRelatedAlso(ClassDef *cd) = 0;
 
     // enumeration specific members
-    virtual void insertEnumField(const MemberDef *md) = 0;
-    virtual void setEnumScope(const MemberDef *md,bool livesInsideEnum=FALSE) = 0;
-    virtual void setEnumClassScope(const ClassDef *cd) = 0;
+    virtual void insertEnumField(MemberDef *md) = 0;
+    virtual void setEnumScope(MemberDef *md,bool livesInsideEnum=FALSE) = 0;
+    virtual void setEnumClassScope(ClassDef *cd) = 0;
     virtual void setDocumentedEnumValues(bool value) = 0;
     virtual void setAnonymousEnumType(const MemberDef *md) = 0;
 
@@ -361,7 +369,7 @@ class MemberDefMutable : public DefinitionMutable, public MemberDef
     virtual void setAccessorType(ClassDef *cd,const QCString &t) = 0;
 
     // namespace related members
-    virtual void setNamespace(const NamespaceDef *nd) = 0;
+    virtual void setNamespace(NamespaceDef *nd) = 0;
 
     // member group related members
     virtual void setMemberGroupId(int id) = 0;
@@ -370,14 +378,17 @@ class MemberDefMutable : public DefinitionMutable, public MemberDef
     // anonymous scope members
     virtual void setFromAnonymousMember(MemberDef *m) = 0;
 
-    virtual void enableCallGraph(bool e) = 0;
-    virtual void enableCallerGraph(bool e) = 0;
-
-    virtual void enableReferencedByRelation(bool e) = 0;
-    virtual void enableReferencesRelation(bool e) = 0;
+    virtual void overrideCallGraph(bool e) = 0;
+    virtual void overrideCallerGraph(bool e) = 0;
+    virtual void overrideReferencedByRelation(bool e) = 0;
+    virtual void overrideReferencesRelation(bool e) = 0;
+    virtual void overrideInlineSource(bool e) = 0;
+    virtual void overrideEnumValues(bool e) = 0;
 
     virtual void setTemplateMaster(const MemberDef *mt) = 0;
-    virtual void addListReference(Definition *d) = 0;
+    virtual void setFormalTemplateArguments(const ArgumentList &al) = 0;
+    virtual void addListReference(const Definition *) = 0;
+    virtual void addRequirementReferences(const Definition *) = 0;
     virtual void setDocsForDefinition(bool b) = 0;
     virtual void setGroupAlias(const MemberDef *md) = 0;
 
@@ -395,14 +406,15 @@ class MemberDefMutable : public DefinitionMutable, public MemberDef
     virtual void setCategory(ClassDef *) = 0;
     virtual void setCategoryRelation(const MemberDef *) = 0;
 
-    virtual void setDocumentation(const QCString &d,const QCString &docFile,int docLine,bool stripWhiteSpace=TRUE) = 0;
-    virtual void setBriefDescription(const QCString &b,const QCString &briefFile,int briefLine) = 0;
-    virtual void setInbodyDocumentation(const QCString &d,const QCString &inbodyFile,int inbodyLine) = 0;
-
-    virtual void setHidden(bool b) = 0;
-
     virtual void setRequiresClause(const QCString &req) = 0;
 
+    virtual void addQualifiers(const StringVector &qualifiers) = 0;
+
+    virtual void setModuleDef(ModuleDef *mod) = 0;
+
+    // macro redefinition
+    virtual void setRedefineCount(int count) = 0;
+
     //-----------------------------------------------------------------------------------
     // --- actions ----
     //-----------------------------------------------------------------------------------
@@ -410,6 +422,8 @@ class MemberDefMutable : public DefinitionMutable, public MemberDef
     virtual void findSectionsInDocumentation() = 0;
     //virtual void addToSearchIndex() const = 0;
 
+    virtual ClassDefMutable *getClassDefMutable() = 0;
+
     //-----------------------------------------------------------------------------------
     // --- write output ----
     //-----------------------------------------------------------------------------------
@@ -419,21 +433,12 @@ class MemberDefMutable : public DefinitionMutable, public MemberDef
                  bool inGroup,bool showEnumValues=FALSE,bool
                  showInline=FALSE) const = 0;
     virtual void writeMemberDocSimple(OutputList &ol,const Definition *container) const = 0;
-    virtual void writeTagFile(TextStream &) const = 0;
-    virtual void writeLink(OutputList &ol,
-                 const ClassDef *cd,const NamespaceDef *nd,const FileDef *fd,const GroupDef *gd,
-                 bool onlyText=FALSE) const = 0;
+    virtual void writeTagFile(TextStream &,bool useQualifiedName,bool showNamespaceMembers) const = 0;
 
     // write helpers
-    virtual void setAnonymousUsed() const = 0;
-    virtual void setFromAnonymousScope(bool b) const = 0;
-
+    virtual void setFromAnonymousScope(bool b) = 0;
 };
 
-inline ClassDefMutable *MemberDef::getClassDefMutable() const
-{
-  return toClassDefMutable(getClassDef());
-}
 
 // --- Cast functions
 
@@ -441,21 +446,20 @@ MemberDef            *toMemberDef(Definition *d);
 MemberDef            *toMemberDef(DefinitionMutable *d);
 const MemberDef      *toMemberDef(const Definition *d);
 MemberDefMutable     *toMemberDefMutable(Definition *d);
-MemberDefMutable     *toMemberDefMutable(const Definition *d);
 
 //------------------------------------------------------------------------
 
 
 /** Factory method to create a new instance of a MemberDef */
-MemberDefMutable *createMemberDef(const QCString &defFileName,int defLine,int defColumn,
+std::unique_ptr<MemberDef> createMemberDef(const QCString &defFileName,int defLine,int defColumn,
               const QCString &type,const QCString &name,const QCString &args,
               const QCString &excp,Protection prot,Specifier virt,bool stat,
               Relationship related,MemberType t,const ArgumentList &tal,
               const ArgumentList &al,const QCString &metaData);
 
-MemberDef *createMemberDefAlias(const Definition *newScope,const MemberDef *aliasMd);
+std::unique_ptr<MemberDef> createMemberDefAlias(const Definition *newScope,const MemberDef *aliasMd);
 
 void combineDeclarationAndDefinition(MemberDefMutable *mdec,MemberDefMutable *mdef);
-void addDocCrossReference(MemberDefMutable *src,MemberDefMutable *dst);
+void addDocCrossReference(const MemberDef *src,const MemberDef *dst);
 
 #endif
