@@ -59,10 +59,15 @@ class ModuleDefImpl : public DefinitionMixin<ModuleDef>
     QCString displayName(bool=TRUE) const override { return name(); }
     QCString getOutputFileBase() const override;
     QCString anchor() const override { return ""; }
-    bool isLinkableInProject() const override { return isLinkable() && !isHidden() && !isReference(); }
-    bool isLinkable() const override { return hasDocumentation(); }
+    bool isLinkableInProject() const override {
+      if (m_primaryInterface) return m_primaryInterface->isLinkableInProject();
+      else return isLinkable() && !isHidden() && !isReference(); }
+    bool isLinkable() const override {
+      if (m_primaryInterface) return m_primaryInterface->isLinkable();
+      else return hasDocumentation(); }
     QCString qualifiedName() const override;
     void writeSummaryLinks(OutputList &ol) const override;
+    void writePageNavigation(OutputList &ol) const override;
 
     // --- ModuleDef
     Type moduleType() const override { return m_type; }
@@ -110,12 +115,12 @@ class ModuleDefImpl : public DefinitionMixin<ModuleDef>
     void addMemberToList(MemberListType lt,MemberDef *md);
     void addExportedModule(const QCString &moduleName,const ImportInfo &info);
     void addListReferences();
+    void addRequirementReferences();
     void addMembersToMemberGroup();
     void distributeMemberGroupDocumentation();
     void findSectionsInDocumentation();
     void sortMemberLists();
 
-    //ModuleMap &partitions() { return m_partitions; }
     void mergeSymbolsFrom(ModuleDefImpl *other);
     bool hasDetailedDescription() const;
     void countMembers();
@@ -155,7 +160,7 @@ void ModuleDefImpl::addPartition(ModuleDefImpl *mod)
   std::string qName = mod->qualifiedName().str();
   if (m_partitions.find(qName)==m_partitions.end())
   {
-    m_partitions.insert(std::make_pair(qName,mod));
+    m_partitions.emplace(qName,mod);
   }
 }
 
@@ -185,13 +190,13 @@ void ModuleDefImpl::addHeader(int line,const QCString &headerName,bool isSystem)
 void ModuleDefImpl::addImport(int line,const QCString &moduleName,const QCString &partitionName,bool isExported)
 {
   AUTO_TRACE("name={}:line={},module={},partition={}",name(),line,moduleName,partitionName);
-  m_imports.insert(std::make_pair(getDefFileName().str()+":"+std::to_string(line),ImportInfo(this,moduleName,line,partitionName,isExported)));
+  m_imports[getDefFileName().str()+":"+std::to_string(line)].push_back(ImportInfo(this,moduleName,line,partitionName,isExported));
 }
 
 void ModuleDefImpl::addExportedModule(const QCString &moduleName,const ImportInfo &info)
 {
   AUTO_TRACE("name={}:moduleName={},import={}",name(),moduleName,info.importName);
-  m_exportedModules.insert(std::make_pair(moduleName.str(),info));
+  m_exportedModules[moduleName.str()].push_back(info);
 }
 
 void ModuleDefImpl::addClassToModule(const Entry *root,ClassDef *cd)
@@ -335,15 +340,25 @@ void ModuleDefImpl::mergeSymbolsFrom(ModuleDefImpl *other)
 void ModuleDefImpl::writeDocumentation(OutputList &ol)
 {
   if (isReference()) return;
+  bool generateTreeView = Config_getBool(GENERATE_TREEVIEW);
   ol.pushGeneratorState();
   AUTO_TRACE("%s file=%s",name(),getDefFileName());
   SrcLangExt lang = getLanguage();
-  QCString pageTitle = theTranslator->trModuleReference(displayName());
-  startFile(ol,getOutputFileBase(),name(),pageTitle,HighlightedItem::ModuleVisible,false,QCString(),0);
+  QCString pageTitle;
+  if (Config_getBool(HIDE_COMPOUND_REFERENCE))
+  {
+    pageTitle = displayName();
+  }
+  else
+  {
+    pageTitle = theTranslator->trModuleReference(displayName());
+  }
+  startFile(ol,getOutputFileBase(),false,name(),pageTitle,HighlightedItem::ModuleVisible,false,QCString());
 
   // ---- title part
   ol.startHeaderSection();
-  writeSummaryLinks(ol);
+  bool writeOutlinePanel = generateTreeView && Config_getBool(PAGE_OUTLINE_PANEL);
+  if (!writeOutlinePanel) writeSummaryLinks(ol);
   ol.startTitleHead(getOutputFileBase());
 
   ol.pushGeneratorState();
@@ -463,14 +478,24 @@ void ModuleDefImpl::writeDocumentation(OutputList &ol)
       case LayoutDocEntry::DirSubDirs:
       case LayoutDocEntry::DirFiles:
       case LayoutDocEntry::DirGraph:
-        err("Internal inconsistency: member %d should not be part of "
-            "LayoutDocManager::Module entry list\n",lde->kind());
+        err("Internal inconsistency: member '{}' should not be part of "
+            "LayoutDocManager::Module entry list\n",lde->entryToString());
         break;
     }
   }
 
   //---------------------------------------- end flexible part -------------------------------
-  endFile(ol);
+  if (generateTreeView)
+  {
+    ol.pushGeneratorState();
+    ol.disableAllBut(OutputType::Html);
+    ol.endContents();
+    ol.writeString("</div><!-- doc-content -->\n");
+    writePageNavigation(ol);
+    ol.writeString("</div><!-- container -->\n");
+    ol.popGeneratorState();
+  }
+  endFile(ol,generateTreeView,true);
 
   ol.popGeneratorState();
 }
@@ -525,15 +550,19 @@ void ModuleDefImpl::writeDetailedDescription(OutputList &ol,const QCString &titl
       ol.disableAllBut(OutputType::Html);
       ol.writeAnchor(QCString(),"details");
     ol.popGeneratorState();
-    ol.startGroupHeader();
+    ol.startGroupHeader("details");
     ol.parseText(title);
     ol.endGroupHeader();
 
     ol.startTextBlock();
     if (!briefDescription().isEmpty() && Config_getBool(REPEAT_BRIEF))
     {
-      ol.generateDoc(briefFile(),briefLine(),this,nullptr,briefDescription(),FALSE,FALSE,
-                     QCString(),FALSE,FALSE,Config_getBool(MARKDOWN_SUPPORT));
+      ol.generateDoc(briefFile(),
+                     briefLine(),
+                     this,
+                     nullptr,
+                     briefDescription(),
+                     DocOptions());
     }
     if (!briefDescription().isEmpty() && Config_getBool(REPEAT_BRIEF) &&
         !documentation().isEmpty())
@@ -549,8 +578,13 @@ void ModuleDefImpl::writeDetailedDescription(OutputList &ol,const QCString &titl
     }
     if (!documentation().isEmpty())
     {
-      ol.generateDoc(docFile(),docLine(),this,nullptr,documentation()+"\n",TRUE,FALSE,
-                     QCString(),FALSE,FALSE,Config_getBool(MARKDOWN_SUPPORT));
+      ol.generateDoc(docFile(),
+                     docLine(),
+                     this,
+                     nullptr,
+                     documentation()+"\n",
+                     DocOptions()
+                     .setIndexWords(true));
     }
     ol.endTextBlock();
   }
@@ -562,9 +596,15 @@ void ModuleDefImpl::writeBriefDescription(OutputList &ol)
   {
     auto parser { createDocParser() };
     auto ast    { validatingParseDoc(*parser.get(),
-                                     briefFile(),briefLine(),this,nullptr,
-                                     briefDescription(),TRUE,FALSE,
-                                     QCString(),TRUE,FALSE,Config_getBool(MARKDOWN_SUPPORT)) };
+                                     briefFile(),
+                                     briefLine(),
+                                     this,
+                                     nullptr,
+                                     briefDescription(),
+                                     DocOptions()
+                                     .setIndexWords(true)
+                                     .setSingleLine(true))
+                };
     if (!ast->isEmpty())
     {
       ol.startParagraph();
@@ -609,7 +649,7 @@ void ModuleDefImpl::writeMemberDeclarations(OutputList &ol,MemberListType lt,con
 void ModuleDefImpl::writeMemberDocumentation(OutputList &ol,MemberListType lt,const QCString &title)
 {
   MemberList * ml = getMemberList(lt);
-  if (ml) ml->writeDocumentation(ol,name(),this,title);
+  if (ml) ml->writeDocumentation(ol,name(),this,title,ml->listType().toLabel());
 }
 
 void ModuleDefImpl::writeAuthorSection(OutputList &ol)
@@ -646,16 +686,15 @@ void ModuleDefImpl::countMembers()
 
 void ModuleDefImpl::addListReferences()
 {
-  const RefItemVector &xrefItems = xrefListItems();
-  addRefItem(xrefItems,
-      qualifiedName(),
-      getLanguage()==SrcLangExt::Fortran ?
-      theTranslator->trModule(TRUE,TRUE) :
-      theTranslator->trNamespace(TRUE,TRUE),
-      getOutputFileBase(),displayName(),
-      QCString(),
-      this
-      );
+  addRefItem(xrefListItems(),
+             qualifiedName(),
+             getLanguage()==SrcLangExt::Fortran ?
+             theTranslator->trModule(TRUE,TRUE) :
+             theTranslator->trNamespace(TRUE,TRUE),
+             getOutputFileBase(),displayName(),
+             QCString(),
+             this
+            );
   for (const auto &mg : m_memberGroups)
   {
     mg->addListReferences(this);
@@ -669,6 +708,22 @@ void ModuleDefImpl::addListReferences()
   }
 }
 
+void ModuleDefImpl::addRequirementReferences()
+{
+  RequirementManager::instance().addRequirementRefsForSymbol(this);
+  for (const auto &mg : m_memberGroups)
+  {
+    mg->addRequirementReferences(this);
+  }
+  for (auto &ml : m_memberLists)
+  {
+    if (ml->listType().isDocumentation())
+    {
+      ml->addRequirementReferences(this);
+    }
+  }
+}
+
 void ModuleDefImpl::addMembersToMemberGroup()
 {
   for (auto &ml : m_memberLists)
@@ -818,6 +873,11 @@ void ModuleDefImpl::writeSummaryLinks(OutputList &ol) const
   ol.popGeneratorState();
 }
 
+void ModuleDefImpl::writePageNavigation(OutputList &ol) const
+{
+  ol.writePageOutline();
+}
+
 void ModuleDefImpl::writeDeclarationLink(OutputList &ol,bool &found,const QCString &header,bool localNames) const
 {
   if (isLinkable())
@@ -838,9 +898,11 @@ void ModuleDefImpl::writeDeclarationLink(OutputList &ol,bool &found,const QCStri
       found=TRUE;
     }
     ol.startMemberDeclaration();
-    ol.startMemberItem(anchor(),OutputGenerator::MemberItemType::Normal);
-    ol.writeString("module ");
     QCString cname = displayName(!localNames);
+    QCString anc = anchor();
+    if (anc.isEmpty()) anc=cname; else anc.prepend(cname+"_");
+    ol.startMemberItem(anc,OutputGenerator::MemberItemType::Normal);
+    ol.writeString("module ");
     ol.insertMemberAlign();
     if (isLinkable())
     {
@@ -861,10 +923,15 @@ void ModuleDefImpl::writeDeclarationLink(OutputList &ol,bool &found,const QCStri
     if (!briefDescription().isEmpty() && Config_getBool(BRIEF_MEMBER_DESC))
     {
       auto parser { createDocParser() };
-      auto ast    { validatingParseDoc(
-                                *parser.get(),briefFile(),briefLine(),this,nullptr,
-                                briefDescription(),FALSE,FALSE,
-                                QCString(),TRUE,FALSE,Config_getBool(MARKDOWN_SUPPORT)) };
+      auto ast    { validatingParseDoc(*parser.get(),
+                                       briefFile(),
+                                       briefLine(),
+                                       this,
+                                       nullptr,
+                                       briefDescription(),
+                                       DocOptions()
+                                       .setSingleLine(true))
+                   };
       if (!ast->isEmpty())
       {
         ol.startMemberDescription(anchor());
@@ -886,32 +953,40 @@ void ModuleDefImpl::writeExports(OutputList &ol,const QCString &title)
     ol.parseText(title);
     ol.endMemberHeader();
     ol.startMemberList();
-    for (auto &[moduleName,importInfo] : m_exportedModules)
+    for (const auto &[moduleName,importInfoList] : m_exportedModules)
     {
-      ModuleDef *mod = ModuleManager::instance().getPrimaryInterface(importInfo.importName);
-      ol.startMemberDeclaration();
-      ol.startMemberItem(importInfo.importName,OutputGenerator::MemberItemType::Normal);
-      ol.docify(theTranslator->trModule(FALSE,TRUE)+" ");
-      ol.insertMemberAlign();
-      if (mod && mod->isLinkable())
+      for (const auto &importInfo : importInfoList)
       {
-        ol.writeObjectLink(mod->getReference(),mod->getOutputFileBase(),QCString(),mod->displayName());
+        ModuleDef *mod = ModuleManager::instance().getPrimaryInterface(importInfo.importName);
+        ol.startMemberDeclaration();
+        ol.startMemberItem(importInfo.importName,OutputGenerator::MemberItemType::Normal);
+        ol.docify(theTranslator->trModule(FALSE,TRUE)+" ");
+        ol.insertMemberAlign();
+        if (mod && mod->isLinkable())
+        {
+          ol.writeObjectLink(mod->getReference(),mod->getOutputFileBase(),QCString(),mod->displayName());
+        }
+        else
+        {
+          ol.startBold();
+          ol.docify(importInfo.importName);
+          ol.endBold();
+        }
+        ol.endMemberItem(OutputGenerator::MemberItemType::Normal);
+        if (mod && !mod->briefDescription().isEmpty() && Config_getBool(BRIEF_MEMBER_DESC))
+        {
+          ol.startMemberDescription(mod->getOutputFileBase());
+          ol.generateDoc(briefFile(),
+                         briefLine(),
+                         mod,
+                         nullptr,
+                         mod->briefDescription(),
+                         DocOptions()
+                         .setSingleLine(true));
+          ol.endMemberDescription();
+        }
+        ol.endMemberDeclaration(QCString(),QCString());
       }
-      else
-      {
-        ol.startBold();
-        ol.docify(importInfo.importName);
-        ol.endBold();
-      }
-      ol.endMemberItem(OutputGenerator::MemberItemType::Normal);
-      if (mod && !mod->briefDescription().isEmpty() && Config_getBool(BRIEF_MEMBER_DESC))
-      {
-        ol.startMemberDescription(mod->getOutputFileBase());
-        ol.generateDoc(briefFile(),briefLine(),mod,nullptr,mod->briefDescription(),FALSE,FALSE,
-            QCString(),TRUE,FALSE,Config_getBool(MARKDOWN_SUPPORT));
-        ol.endMemberDescription();
-      }
-      ol.endMemberDeclaration(QCString(),QCString());
     }
     ol.endMemberList();
   }
@@ -932,7 +1007,10 @@ void ModuleDefImpl::writeFiles(OutputList &ol,const QCString &title)
       if (fd)
       {
         ol.startMemberDeclaration();
-        ol.startMemberItem(fd->anchor(),OutputGenerator::MemberItemType::Normal);
+        QCString fname = fd->displayName();
+        QCString anc = fd->anchor();
+        if (anc.isEmpty()) anc=fname; else anc.prepend(fname+"_");
+        ol.startMemberItem(anc,OutputGenerator::MemberItemType::Normal);
         ol.docify(theTranslator->trFile(FALSE,TRUE)+" ");
         ol.insertMemberAlign();
         QCString path=fd->getPath();
@@ -942,7 +1020,7 @@ void ModuleDefImpl::writeFiles(OutputList &ol,const QCString &title)
         }
         if (fd->isLinkable())
         {
-          ol.writeObjectLink(fd->getReference(),fd->getOutputFileBase(),QCString(),fd->displayName());
+          ol.writeObjectLink(fd->getReference(),fd->getOutputFileBase(),QCString(),fname);
         }
         else
         {
@@ -954,8 +1032,13 @@ void ModuleDefImpl::writeFiles(OutputList &ol,const QCString &title)
         if (!fd->briefDescription().isEmpty() && Config_getBool(BRIEF_MEMBER_DESC))
         {
           ol.startMemberDescription(fd->getOutputFileBase());
-          ol.generateDoc(briefFile(),briefLine(),fd,nullptr,fd->briefDescription(),FALSE,FALSE,
-              QCString(),TRUE,FALSE,Config_getBool(MARKDOWN_SUPPORT));
+          ol.generateDoc(briefFile(),
+                         briefLine(),
+                         fd,
+                         nullptr,
+                         fd->briefDescription(),
+                         DocOptions()
+                         .setSingleLine(true));
           ol.endMemberDescription();
         }
         ol.endMemberDeclaration(QCString(),QCString());
@@ -1090,7 +1173,7 @@ void ModuleDefImpl::writeTagFile(TextStream &tagFile) const
       case LayoutDocEntry::MemberDecl:
         {
           const LayoutDocEntryMemberDecl *lmd = dynamic_cast<const LayoutDocEntryMemberDecl*>(lde.get());
-          if (lmd)
+          if (lmd && lmd->visible())
           {
             MemberList * ml = getMemberList(lmd->type);
             if (ml)
@@ -1198,7 +1281,7 @@ void ModuleManager::createModuleDef(const QCString &fileName,int line,int column
   {
     ModuleList ml;
     ml.push_back(mod);
-    p->moduleNameMap.insert(std::make_pair(moduleName.str(),ml));
+    p->moduleNameMap.emplace(moduleName.str(),ml);
   }
   else
   {
@@ -1246,7 +1329,7 @@ void ModuleManager::addImport(const QCString &moduleFile,int line,const QCString
   else // import outside of a module
   {
     AUTO_TRACE_ADD("outside module");
-    p->externalImports.insert(std::make_pair(moduleFile.str(),ImportInfo(nullptr,importName,line,partitionName)));
+    p->externalImports[moduleFile.str()].emplace_back(nullptr,importName,line,partitionName);
   }
 }
 
@@ -1278,7 +1361,7 @@ void ModuleManager::addMemberToModule(const Entry *root,MemberDef *md)
 {
   std::lock_guard lock(p->mutex);
   auto mod = p->moduleFileMap.find(root->fileName);
-  if (mod && root->exported)
+  if (mod && root->exported && md->getClassDef()==nullptr)
   {
     toModuleDefImpl(mod)->addMemberToModule(root,md);
     auto mdm = toMemberDefMutable(md);
@@ -1301,32 +1384,38 @@ void ModuleManager::addTagInfo(const QCString &fileName,const QCString &tagFile,
 void ModuleManager::resolvePartitionsRecursively(ModuleDef *intfMod, ModuleDef *mod)
 {
   AUTO_TRACE();
-  for (auto &[partitionFileName,importInfo] : mod->getImports()) // foreach import
+  for (auto &[partitionFileName,importInfoList] : mod->getImports()) // foreach import
   {
-    AUTO_TRACE_ADD("partitionFileName={} importName={} partitionName={}",
-        partitionFileName,importInfo.importName,importInfo.partitionName);
-    if (importInfo.importName==intfMod->name() && !importInfo.partitionName.isEmpty() &&
-        importInfo.exported) // that is an exported partition of this module
+    for (auto &importInfo : importInfoList)
     {
-      auto it = p->moduleNameMap.find(importInfo.importName.str());
-      if (it != p->moduleNameMap.end())
+      AUTO_TRACE_ADD("partitionFileName={} importName={} partitionName={}",
+          partitionFileName,importInfo.importName,importInfo.partitionName);
+      if (importInfo.importName==intfMod->name() && !importInfo.partitionName.isEmpty() &&
+          importInfo.exported) // that is an exported partition of this module
       {
-        for (auto importedMod : it->second)
+        auto it = p->moduleNameMap.find(importInfo.importName.str());
+        if (it != p->moduleNameMap.end())
         {
-          if (importedMod->qualifiedName()==importInfo.importName+":"+importInfo.partitionName)
+          for (auto importedMod : it->second)
           {
-            AUTO_TRACE_ADD("Interface module {} exports partition {}:{}",
-                mod->name(),importedMod->name(),importedMod->partitionName());
-            toModuleDefImpl(intfMod)->addPartition(toModuleDefImpl(importedMod));
-            toModuleDefImpl(importedMod)->setPrimaryInterface(intfMod);
-            for (const auto &[partitionFileName_,partitionImportInfo] : importedMod->getImports())
+            if (importedMod->qualifiedName()==importInfo.importName+":"+importInfo.partitionName)
             {
-              if (partitionImportInfo.exported && intfMod->name()!=partitionImportInfo.importName)
+              AUTO_TRACE_ADD("Interface module {} exports partition {}:{}",
+                  mod->name(),importedMod->name(),importedMod->partitionName());
+              toModuleDefImpl(intfMod)->addPartition(toModuleDefImpl(importedMod));
+              toModuleDefImpl(importedMod)->setPrimaryInterface(intfMod);
+              for (const auto &[partitionFileName_,partitionImportInfoList] : importedMod->getImports())
               {
-                toModuleDefImpl(intfMod)->addExportedModule(partitionImportInfo.importName,partitionImportInfo);
+                for (const auto &partitionImportInfo : partitionImportInfoList)
+                {
+                  if (partitionImportInfo.exported && intfMod->name()!=partitionImportInfo.importName)
+                  {
+                    toModuleDefImpl(intfMod)->addExportedModule(partitionImportInfo.importName,partitionImportInfo);
+                  }
+                }
               }
+              resolvePartitionsRecursively(intfMod,importedMod);
             }
-            resolvePartitionsRecursively(intfMod,importedMod);
           }
         }
       }
@@ -1345,11 +1434,14 @@ void ModuleManager::resolvePartitions()
     }
 
     // copy exported imports to m_exportedModules
-    for (const auto &[fileName,importInfo] : mod->getImports())
+    for (const auto &[fileName,importInfoList] : mod->getImports())
     {
-      if (importInfo.exported && mod->name()!=importInfo.importName)
+      for (const auto &importInfo : importInfoList)
       {
-        toModuleDefImpl(mod)->addExportedModule(importInfo.importName,importInfo);
+        if (importInfo.exported && mod->name()!=importInfo.importName)
+        {
+          toModuleDefImpl(mod)->addExportedModule(importInfo.importName,importInfo);
+        }
       }
     }
 
@@ -1372,37 +1464,53 @@ void ModuleManager::resolveImports()
     FileDef *fd = mod->getFileDef();
     if (fd)
     {
-      for (const auto &[fileName,importInfo] : mod->getImports())
+      for (const auto &[fileName,importInfoList] : mod->getImports())
       {
-        ModuleDef *importedModule = getPrimaryInterface(importInfo.importName);
-        const FileDef *importedFd = importedModule ? importedModule->getFileDef() : nullptr;
-        AUTO_TRACE_ADD("module: addIncludeDependency {}->{}:{} fd={}",
-             mod->qualifiedName(), importInfo.qualifiedName(), importInfo.line, fd?fd->absFilePath():"");
-        fd->addIncludeDependency(importedFd,importInfo.qualifiedName(),IncludeKind::ImportModule);
+        for (const auto &importInfo : importInfoList)
+        {
+          ModuleDef *importedModule = getPrimaryInterface(importInfo.importName);
+          const FileDef *importedFd = importedModule ? importedModule->getFileDef() : nullptr;
+          AUTO_TRACE_ADD("module: addIncludeDependency {}->{}:{} fd={}",
+              mod->qualifiedName(), importInfo.qualifiedName(), importInfo.line, fd?fd->absFilePath():"");
+          fd->addIncludeDependency(importedFd,importInfo.qualifiedName(),IncludeKind::ImportModule);
+        }
       }
     }
   }
-  for (const auto &[fileName,importInfo] : p->externalImports)
+  for (const auto &[fileName,importInfoList] : p->externalImports)
   {
-    bool ambig = false;
-    FileDef *fd = findFileDef(Doxygen::inputNameLinkedMap,QCString(fileName),ambig);
-    if (fd)
+    for (const auto &importInfo : importInfoList)
     {
-      ModuleDef *mod = getPrimaryInterface(importInfo.importName);
-      FileDef *importedFd = mod ? mod->getFileDef() : nullptr;
-      fd->addIncludeDependency(importedFd,importInfo.importName,IncludeKind::ImportModule);
+      bool ambig = false;
+      FileDef *fd = findFileDef(Doxygen::inputNameLinkedMap,fileName,ambig);
+      AUTO_TRACE_ADD("externalImport name={} fd={}",fileName,(void*)fd);
+      if (fd)
+      {
+        ModuleDef *mod = getPrimaryInterface(importInfo.importName);
+        FileDef *importedFd = mod ? mod->getFileDef() : nullptr;
+        fd->addIncludeDependency(importedFd,importInfo.importName,IncludeKind::ImportModule);
+        if (importedFd)
+        {
+          importedFd->addIncludedByDependency(fd,stripFromPath(fileName),IncludeKind::ImportModule);
+        }
+      }
     }
   }
   for (const auto &headerInfo : p->headers)
   {
     bool ambig = false;
     FileDef *fd = findFileDef(Doxygen::inputNameLinkedMap,headerInfo.fileName,ambig);
+    AUTO_TRACE_ADD("header name={} fd={}",headerInfo.fileName,(void*)fd);
     if (fd)
     {
       QCString resolvedHeader = determineAbsoluteIncludeName(headerInfo.fileName,headerInfo.headerName);
       FileDef *importFd = findFileDef(Doxygen::inputNameLinkedMap,resolvedHeader,ambig);
       fd->addIncludeDependency(importFd, headerInfo.headerName,
                                headerInfo.isSystem ? IncludeKind::ImportSystem : IncludeKind::ImportLocal);
+      if (importFd)
+      {
+        importFd->addIncludedByDependency(fd,headerInfo.fileName,IncludeKind::ImportModule);
+      }
     }
   }
 }
@@ -1468,7 +1576,7 @@ void ModuleManager::writeDocumentation(OutputList &ol)
 int ModuleManager::numDocumentedModules() const
 {
   int count=0;
-  for (auto &mod : p->moduleFileMap) // foreach module
+  for (const auto &mod : p->moduleFileMap) // foreach module
   {
     if (mod->isPrimaryInterface()) count++;
   }
@@ -1480,14 +1588,19 @@ const ModuleLinkedMap &ModuleManager::modules() const
   return p->moduleFileMap;
 }
 
+ModuleLinkedMap &ModuleManager::modules()
+{
+  return p->moduleFileMap;
+}
+
 void ModuleManager::addDocs(const Entry *root)
 {
   AUTO_TRACE("file={} module={}",root->fileName,root->name);
   if (root->doc.isEmpty() && root->brief.isEmpty()) return;
   if (root->name.find(':')!=-1)
   {
-    warn(root->fileName,root->startLine,"Ignoring documentation for module partition %s. Please place documentation at the primary module name",
-        qPrint(root->name));
+    warn(root->fileName,root->startLine,"Ignoring documentation for module partition {}. Please place documentation at the primary module name",
+        root->name);
   }
   else
   {
@@ -1503,17 +1616,18 @@ void ModuleManager::addDocs(const Entry *root)
         mod->setHidden(root->hidden);
         mod->setBodySegment(root->startLine,root->bodyLine,root->endBodyLine);
         mod->setRefItems(root->sli);
+        mod->setRequirementReferences(root->rqli);
         //mod->addSectionsToDefinition(root->anchors);
         addModuleToGroups(root,mod);
       }
       else
       {
-        warn(root->fileName,root->startLine,"Found documentation for module %s but it has no primary interface unit.",qPrint(root->name));
+        warn(root->fileName,root->startLine,"Found documentation for module {} but it has no primary interface unit.",root->name);
       }
     }
     else
     {
-      warn(root->fileName,root->startLine,"Found documentation for unknown module %s.",qPrint(root->name));
+      warn(root->fileName,root->startLine,"Found documentation for unknown module {}.",root->name);
     }
   }
 }
@@ -1523,7 +1637,7 @@ ModuleDef *ModuleManager::getPrimaryInterface(const QCString &moduleName) const
   auto it = p->moduleNameMap.find(moduleName.str());
   if (it != p->moduleNameMap.end())
   {
-    for (auto &mod : it->second)
+    for (const auto &mod : it->second)
     {
       if (mod->isPrimaryInterface())
       {
@@ -1536,15 +1650,23 @@ ModuleDef *ModuleManager::getPrimaryInterface(const QCString &moduleName) const
 
 void ModuleManager::addListReferences()
 {
-  for (auto &mod : p->moduleFileMap) // foreach module
+  for (const auto &mod : p->moduleFileMap) // foreach module
   {
     if (mod->isPrimaryInterface()) toModuleDefImpl(mod)->addListReferences();
   }
 }
 
+void ModuleManager::addRequirementReferences()
+{
+  for (const auto &mod : p->moduleFileMap) // foreach module
+  {
+    if (mod->isPrimaryInterface()) toModuleDefImpl(mod)->addRequirementReferences();
+  }
+}
+
 void ModuleManager::addMembersToMemberGroup()
 {
-  for (auto &mod : p->moduleFileMap) // foreach module
+  for (const auto &mod : p->moduleFileMap) // foreach module
   {
     if (mod->isPrimaryInterface()) toModuleDefImpl(mod)->addMembersToMemberGroup();
   }
@@ -1552,7 +1674,7 @@ void ModuleManager::addMembersToMemberGroup()
 
 void ModuleManager::distributeMemberGroupDocumentation()
 {
-  for (auto &mod : p->moduleFileMap) // foreach module
+  for (const auto &mod : p->moduleFileMap) // foreach module
   {
     if (mod->isPrimaryInterface()) toModuleDefImpl(mod)->distributeMemberGroupDocumentation();
   }
