@@ -548,14 +548,13 @@ void VhdlDocGen::findAllArchitectures(std::vector<QCString>& qll,const ClassDef
 
 const ClassDef* VhdlDocGen::findArchitecture(const ClassDef *cd)
 {
-  QCString nn=cd->name();
   for (const auto &citer : *Doxygen::classLinkedMap)
   {
     QCString jj=citer->name();
     StringVector ql=split(jj.str(),":");
     if (ql.size()>1)
     {
-      if (QCString(ql[0])==nn)
+      if (ql[0]==cd->name())
       {
         return citer.get();
       }
@@ -688,7 +687,7 @@ QCString VhdlDocGen::getIndexWord(const QCString &c,int index)
 
   if (index < static_cast<int>(ql.size()))
   {
-    return QCString(ql[index]);
+    return ql[index];
   }
 
   return "";
@@ -1661,10 +1660,14 @@ void VhdlDocGen::writeVHDLDeclaration(MemberDefMutable* mdef,OutputList &ol,
   {
     QCString s=mdef->briefDescription();
     ol.startMemberDescription(mdef->anchor(), QCString(), mm == VhdlSpecifier::PORT);
-    ol.generateDoc(mdef->briefFile(),mdef->briefLine(),
+    ol.generateDoc(mdef->briefFile(),
+                   mdef->briefLine(),
                    mdef->getOuterScope()?mdef->getOuterScope():d,
-                   mdef,s,TRUE,FALSE,
-                   QCString(),TRUE,FALSE);
+                   mdef,
+                   s,
+                   DocOptions()
+                   .setIndexWords(true)
+                   .setSingleLine(true));
     if (detailsVisible)
     {
       ol.pushGeneratorState();
@@ -1758,8 +1761,13 @@ void VhdlDocGen::writeVHDLDeclarations(const MemberList* ml,OutputList &ol,
   if (!subtitle.isEmpty())
   {
     ol.startMemberSubtitle();
-    ol.generateDoc("[generated]",-1,nullptr,nullptr,subtitle,FALSE,FALSE,
-                   QCString(),TRUE,FALSE);
+    ol.generateDoc("[generated]",
+                   -1,
+                   nullptr,
+                   nullptr,
+                   subtitle,
+                   DocOptions()
+                   .setSingleLine(true));
     ol.endMemberSubtitle();
   } //printf("memberGroupList=%p\n",memberGroupList);
 
@@ -1783,8 +1791,12 @@ void VhdlDocGen::writeVHDLDeclarations(const MemberList* ml,OutputList &ol,
       {
         //printf("Member group has docs!\n");
         ol.startMemberGroupDocs();
-        ol.generateDoc("[generated]",-1,nullptr,nullptr,mg->documentation()+"\n",FALSE,FALSE,
-            QCString(),FALSE,FALSE);
+        ol.generateDoc("[generated]",
+                       -1,
+                       nullptr,
+                       nullptr,
+                       mg->documentation()+"\n",
+                       DocOptions());
         ol.endMemberGroupDocs();
       }
       ol.startMemberGroup();
@@ -1865,20 +1877,18 @@ void VhdlDocGen::writeSource(const MemberDef* mdef,OutputList& ol,const QCString
   ol.pushGeneratorState();
   auto &codeOL = ol.codeGenerators();
   codeOL.startCodeFragment("DoxyCode");
-  intf->parseCode(     codeOL,           // codeOutIntf
-                       QCString(),       // scope
-                       codeFragment,     // input
-                       SrcLangExt::VHDL,  // lang
-                       Config_getBool(STRIP_CODE_COMMENTS),
-                       FALSE,            // isExample
-                       QCString(),       // exampleName
-                       mdef->getFileDef(), // fileDef
-                       mdef->getStartBodyLine(),      // startLine
-                       mdef->getEndBodyLine(),        // endLine
-                       TRUE,             // inlineFragment
-                       mdef,             // memberDef
-                       TRUE              // show line numbers
-                      );
+  intf->parseCode(codeOL,           // codeOutIntf
+                  QCString(),       // scope
+                  codeFragment,     // input
+                  SrcLangExt::VHDL,  // lang
+                  Config_getBool(STRIP_CODE_COMMENTS),
+                  CodeParserOptions()
+                  .setFileDef(mdef->getFileDef())
+                  .setStartLine(mdef->getStartBodyLine())
+                  .setEndLine(mdef->getEndBodyLine())
+                  .setInlineFragment(true)
+                  .setMemberDef(mdef)
+                 );
 
   codeOL.endCodeFragment("DoxyCode");
   ol.popGeneratorState();
@@ -2119,7 +2129,7 @@ QCString  VhdlDocGen::parseForBinding(QCString & entity,QCString & arch)
   {
     arch=ql[2];
   }
-  return QCString(label);
+  return label;
 }
 
 
@@ -2296,7 +2306,7 @@ void VhdlDocGen::writeRecUnitDocu(
 
   for(size_t i=0;i<len;i++)
   {
-    QCString n=QCString(ql[i]);
+    QCString n = ql[i];
     ol.startParameterType(first,"");
     ol.endParameterType();
     ol.startParameterName(TRUE);
@@ -2376,7 +2386,7 @@ void VhdlDocGen::addBaseClass(ClassDef* cd,ClassDef *ent)
       VhdlDocGen::deleteAllChars(r,'(');
       r.setNum(r.toInt()+1);
       reg::replace(t, reg, r.str());
-      s.append(t.c_str());
+      s.append(t);
       bcd.usedName=s;
       bcd.templSpecifiers=t;
     }
@@ -2633,15 +2643,15 @@ void FlowChart::printNode(const FlowChart& flo)
     }
     if (flo.type & EMPTNODE)
     {
-      printf("\n NO: %s%s[%d,%d]",q.c_str(),FlowChart::getNodeType(flo.type),flo.stamp,flo.id);
+      printf("\n NO: %s%s[%d,%d]",qPrint(q),FlowChart::getNodeType(flo.type),flo.stamp,flo.id);
     }
     else if (flo.type & COMMENT_NO)
     {
-      printf("\n NO: %s%s[%d,%d]",t.c_str(),FlowChart::getNodeType(flo.type),flo.stamp,flo.id);
+      printf("\n NO: %s%s[%d,%d]",qPrint(t),FlowChart::getNodeType(flo.type),flo.stamp,flo.id);
     }
     else
     {
-      printf("\n NO: %s[%d,%d]",t.c_str(),flo.stamp,flo.id);
+      printf("\n NO: %s[%d,%d]",qPrint(t),flo.stamp,flo.id);
     }
   }
 }
@@ -3009,8 +3019,11 @@ void  FlowChart::printUmlTree()
   QCString htmlOutDir = Config_getString(HTML_OUTPUT);
 
   QCString n=convertNameToFileName();
-  n=PlantumlManager::instance().writePlantUMLSource(htmlOutDir,n,qcs,PlantumlManager::PUML_SVG,"uml",n,1,true);
-  PlantumlManager::instance().generatePlantUMLOutput(n,htmlOutDir,PlantumlManager::PUML_SVG);
+  auto baseNameVector=PlantumlManager::instance().writePlantUMLSource(htmlOutDir,n,qcs,PlantumlManager::PUML_SVG,"uml",n,1,true);
+  for (const auto &baseName: baseNameVector)
+  {
+    PlantumlManager::instance().generatePlantUMLOutput(baseName,htmlOutDir,PlantumlManager::PUML_SVG);
+  }
 }
 
 QCString FlowChart::convertNameToFileName()
